#include "ImportExport.h"

using namespace std;

/*
 *
 */
bool exportLayout(QString filename, std::vector<std::vector<Key *>> keys,
        QString &errorString)
{
    int cols = keys.size();
    if (cols == 0)
        throw runtime_error("error: asked to export layout with no columns");
    int rows = keys[0].size();

    QFile f(filename);
    if (!f.open(QFile::WriteOnly | QFile::Text))
    {
        errorString = f.errorString();
        return false;
    }

    QTextStream ts(&f);
    ts.setFieldAlignment(QTextStream::AlignRight);
    ts.setIntegerBase(16);
    ts.setPadChar('0');
    for (int row = 0; row < rows; row++)
    {
        for (int col = 0; col < cols; col++)
        {
            ts << "0x" << qSetFieldWidth(2) <<
                (unsigned int)keys[col][row]->scancode();
            ts << qSetFieldWidth(1);
            ts << ((col < cols - 1) ? ' ' : '\n');
        }
    }

    f.close();

    return true;
}

/*
 *
 */
bool importLayout(QString filename,
        std::vector<std::vector<unsigned char>> &scancodes,
        QString &errorString)
{
    QFile f(filename);
    if (!f.open(QFile::ReadOnly | QFile::Text))
    {
        errorString = f.errorString();
        return false;
    }

    QTextStream ls(&f);
    QString line;
    int row = 0;
    while (!(line = ls.readLine()).isNull())
    {
        QTextStream ts(&line);
        int col = 0;
        while (!ts.atEnd())
        {
            int scancode;
            ts >> scancode;

            if (row == 0)
                scancodes.push_back(vector<unsigned char>());
            try
            {
                scancodes.at(col++).push_back(scancode);
            }
            catch (std::out_of_range &e)
            {
                errorString = "unequal number of columns";
                return false;
            }
        }
        row++;
    }

    size_t cols = scancodes.size();
    if (cols == 0)
    {
        errorString = "no columns imported";
        return false;
    }

    size_t rows = scancodes[0].size();
    if (rows == 0)
    {
        errorString = "no rows imported";
        return false;
    }

    for (size_t i = 1; i < scancodes.size(); i++)
    {
        if (scancodes[i].size() != rows)
        {
            errorString = "unequal number of rows per column";
            return false;
        }
    }

    return true;
}
